//************************************************************
//
// RdbDefines.h
//
/** @file RdbDefines.h
*
* @brief  Public interfaces for Telemotive Reference Database access
*/
//*****************************************************************

#ifndef RDBDEFINES_H
#define RDBDEFINES_H

#ifdef _MSC_VER

#define WINAPI __stdcall
// always compile the clientlib stuff on Windows
#undef ATOM_CLIENTLIB
#define ATOM_CLIENTLIB 1

#else

#define WINAPI
#include <atom-config.h>	// for ATOM_CLIENTLIB define
#ifndef ATOM_CLIENTLIB
#define ATOM_CLIENTLIB	0
#endif

#endif


// sys
#include <cstdlib>
#include <stdint.h>

#if ATOM_CLIENTLIB

enum RdbEventType
{
    UNKNOWN		= 0,
    STARTUP		= 0x01,	///< bp2 startup
    SHUTDOWN		= 0x02,	///< bp2 shutdown
    MARKER		= 0x03,	///< Marker set
    INFO		= 0x05,	///< Info is set
    SLAVE_OFFSET	= 0x06,	///< cascading slave offset
    SLAVE_TO_MASTER	= 0x07,	///< cascading slave to master
    DATA_DELETED	= 0x08,	///< All data and data space is deleted
    TIME_SET		= 0x09,	///< bp2 time was set
    NEW_TIME		= 0x0A,	///< new time
    SUDDEN_DEATH	= 0x0B,	///< no "real" shutdown was found after startup.
    TSL_SLAVE_OFFSET	= 0x0C,	///< slave is synced with master.
    TSL_SLAVE_TO_MASTER	= 0x0D, ///< slave is not synced with master.
    TSL_SESSION_START	= 0x0E,	///< start of a tsl synchronized session
    TSL_SESSION_END	= 0x0F,	///< end of a tsl synchronized session
    CONFIG		= 0x10,	///< configuration has been updated
    WAKEUP		= 0x11	///< bpng wake-up source
};


/// Interface to an RDB event
struct IRdbEvent
{
    /// Get type of event.
    virtual RdbEventType WINAPI getType() const = 0;

    /// Returns the unique entry ID that can be set to DataSpans for data download and conversion
    virtual uint64_t WINAPI getUniqueId() const = 0;

    /// Returns the event's time stamp in usec since 01.01.1970 UTC
    virtual uint64_t WINAPI getTimeStamp() const = 0;

    /// Returns the logger's time zone that was active at the event's time stamp
    virtual const char* WINAPI getTimeZone() const = 0;

    /// Returns the index of this event. Only used for marker events
    virtual int WINAPI getIndex() const = 0;

    /// Returns additional information. The meaning of this string depends on the event's type. See RDB specification document for more information.
    virtual const char* WINAPI getComment() const = 0;
};


/// Interface to a list of rdb events
struct IRdbEventList
{
    /// DTOR
    virtual ~IRdbEventList() {}

    /// Returns the size of the event list
    virtual size_t WINAPI getSize() const = 0;

    /// Returns a pointer to the IRdbEvent at index
    virtual const IRdbEvent* WINAPI getEvent(size_t index) const = 0;
};


struct IRdbTraceBlock
{
    virtual uint64_t WINAPI getUniqueId() const = 0;
    virtual uint64_t WINAPI getStartTimeStamp() const = 0;
    virtual uint64_t WINAPI getEndTimeStamp() const = 0;
    virtual const char* WINAPI getTimeZone() const = 0;
    virtual const char* WINAPI getLoggerModuleName() const = 0;
    virtual const char* WINAPI getFilePath() const = 0;
    virtual const char* WINAPI getFileName() const = 0;
    virtual uint64_t WINAPI getDataFileSize() const = 0;
    virtual uint64_t WINAPI getDataSize() const = 0;
    virtual uint64_t WINAPI getBlockNumber() const = 0;
    virtual const char* WINAPI getCfgBackupFile() const = 0;
    virtual const char* WINAPI getDataColumnValue(const char *columnName) = 0;
    virtual const char* WINAPI getComment() const = 0;
};


struct IRdbTraceBlockList
{
    /// DTOR
    virtual ~IRdbTraceBlockList() {}

    /// Returns the size of the event list
    virtual size_t WINAPI getSize() const = 0;

    /// Returns a pointer to the IRdbEvent at index
    virtual const IRdbTraceBlock* WINAPI getTraceBlock(size_t index) const = 0;
};

#else	// ATOM_CLIENTLIB

struct IRdbEvent {};
struct IRdbEventList {};
struct IRdbTraceBlock {};
struct IRdbTraceBlockList {};

#endif	// ATOM_CLIENTLIB

#endif	// RDBDEFINES_H
